//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2006
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// Code based on Mozilla.org's GPL code (Mozilla Composer)
// 
// END FLOCK GPL
//

const webPersist = Components.interfaces.nsIWebBrowserPersist;
const nsIFilePicker = Components.interfaces.nsIFilePicker;

var gPrefsService;
var gPrefsBranch;
var gIOService;

function GetPrefsService()
{
  if (gPrefsService)
    return gPrefsService;

  try {
    gPrefsService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
  }
  catch(ex) {
    dump("failed to get prefs service!\n");
  }

  return gPrefsService;
}

function GetPrefs()
{
  if (gPrefsBranch)
    return gPrefsBranch;

  try {
    var prefService = GetPrefsService();
    if (prefService)
      gPrefsBranch = prefService.getBranch(null);

    if (gPrefsBranch)
      return gPrefsBranch;
    else
      dump("failed to get root prefs!\n");
  }
  catch(ex) {
    dump("failed to get root prefs!\n");
  }
  return null;
}

function GetIOService()
{
  if (gIOService)
    return gIOService;

  gIOService = Components.classes["@mozilla.org/network/io-service;1"]
               .getService(Components.interfaces.nsIIOService);

  return gIOService;
}

function GetFileProtocolHandler()
{
  var ios = GetIOService();
  var handler = ios.getProtocolHandler("file");
  return handler.QueryInterface(Components.interfaces.nsIFileProtocolHandler);
}

function FileToString(aFile) {
    var instream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);    
    var scriptinstream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);    
    instream.init(aFile, 0x01, 00400 | 00200 | 00040 | 00004, 0);
    scriptinstream.init(instream);

    //  nsIFileInputSteam is not i18n-aware
    var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    converter.charset = "UTF-8";

    var result = scriptinstream.read(scriptinstream.available());
    var finalresult;
    finalresult = converter.ConvertToUnicode(result);
    finalresult += converter.Finish();
    scriptinstream.close();

    return finalresult;
}

// Set initial directory for a filepicker from URLs saved in prefs
function SetFilePickerDirectory(filePicker)
{
  if (filePicker)
  {
    try {
      var prefBranch = GetPrefs();
      if (prefBranch)
      {
        // Save current directory so we can reset it in SaveFilePickerDirectory
        gFilePickerDirectory = filePicker.displayDirectory;

        var location = prefBranch.getComplexValue("flock.blog.defaultSaveLocation", Components.interfaces.nsILocalFile);
        if (location)
          filePicker.displayDirectory = location;
      }
    }
    catch(e) {}
  }
}

// Save the directory of the selected file to prefs
function SaveFilePickerDirectory(filePicker)
{
  if (filePicker && filePicker.file)
  {
    try {
      var prefBranch = GetPrefs();

      var fileDir;
      if (filePicker.file.parent)
        fileDir = filePicker.file.parent.QueryInterface(Components.interfaces.nsILocalFile);

      if (prefBranch)
       prefBranch.setComplexValue("flock.blog.defaultSaveLocation", Components.interfaces.nsILocalFile, fileDir);
    
     var prefsService = GetPrefsService();
        prefsService.savePrefFile(null);
    } catch (e) { alert(e + " " + e.lineNumber + " " + e.fileName); }
  }
}

function OutputFile(aContent, aFile)
{
   try {
       var obj_Transport = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
       obj_Transport.init(aFile, 0x04 | 0x08, 
                          00400 | 00200 | 00040 | 00004, 0);
       // nsIFileoutputStream is not i18n-aware
       var charset = "UTF-8";  
       var os = Components.classes["@mozilla.org/intl/converter-output-stream;1"].createInstance(Components.interfaces.nsIConverterOutputStream);
       os.init (obj_Transport, charset, 0, 0x0000);
       os.writeString(aContent);
       os.close(); 

       obj_Transport.close();
   }
   catch(e){
       alert(e);
   }

  return true;
}

function PromptToOpenFile()
{
    var localeService = Components.classes["@mozilla.org/intl/nslocaleservice;1"].getService(Components.interfaces.nsILocaleService);
    var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
    var stringBundle = stringBundleService.createBundle("chrome://flock/locale/blog/blog.properties", localeService.getApplicationLocale());
    var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);

    fp.init(window, stringBundle.GetStringFromName("flock.blog.discIO.selectFile"), nsIFilePicker.modeOpen);

    SetFilePickerDirectory(fp, "html");

    fp.displayDirectory = flock.blog.getSaveLocation();

    fp.appendFilters(nsIFilePicker.filterHTML);
    fp.appendFilters(nsIFilePicker.filterText);
    fp.appendFilters(nsIFilePicker.filterAll);

    var res;
    try {
      res = fp.show();
    }
    catch (ex) {
      dump("filePicker.chooseInputFile threw an exception\n");
    }

    if (res == nsIFilePicker.returnOK){
        return fp;
    }
    else {
        return null;
    }
}

function PromptForSaveLocation()
 {
   var dialogResult = {};
   dialogResult.filepickerClick = nsIFilePicker.returnCancel;
   dialogResult.resultingURI = "";
   dialogResult.resultingLocalFile = null;
 
   var fp = null;
   try {
     fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
   } catch (e) {}
   if (!fp) return dialogResult;
 
   var localeService = Components.classes["@mozilla.org/intl/nslocaleservice;1"].getService(Components.interfaces.nsILocaleService);
   var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
   var stringBundle = stringBundleService.createBundle("chrome://flock/locale/blog/blog.properties", localeService.getApplicationLocale());
   fp.init(window, stringBundle.GetStringFromName("flock.blog.discIO.filePickerSaveTitle"), nsIFilePicker.modeSave);
 
   // Set filters according to the type of output
   fp.appendFilters(nsIFilePicker.filterHTML);
   fp.appendFilters(nsIFilePicker.filterAll);
 
   // now let's actually set the filepicker's suggested filename
   var suggestedFileName = $('flock-blog-title').value + '.html';
   if (suggestedFileName)
     fp.defaultString = suggestedFileName;
 
   // set the file picker's current directory
   // assuming we have information needed (like prior saved location)
   try {
     var fileHandler = GetFileProtocolHandler();
     // Initialize to the last-used directory for the particular type (saved in prefs)
     SetFilePickerDirectory(fp);
   }
   catch (e) { alert(e + " " + e.lineNumber + " " + e.fileName); }
 
   dialogResult.filepickerClick = fp.show();
   if (dialogResult.filepickerClick != nsIFilePicker.returnCancel)
   {
     // reset urlstring to new save location
     dialogResult.resultingURIString = fileHandler.getURLSpecFromFile(fp.file);
     dialogResult.resultingLocalFile = fp.file;
     // SaveFilePickerDirectory(fp);
   }
 
   return dialogResult;
 }


// throws an error or returns the file name if user attempted save; false if user canceled save
function SaveDocument(aHtmlContent, aTitle, aFile, aSaveAs)
{
   var urlstring = "";
 
   var mustShowFileDialog = (aSaveAs || (aFile == null));
 
   var replacing = !aSaveAs;
   var titleChanged = false;
   var tempLocalFile = null;
   var theHtmlContent = "<h1>"+aTitle+"</h1>\n"+aHtmlContent; 

   if (mustShowFileDialog)
   {
       try {
           var dialogResult = PromptForSaveLocation();
           if (dialogResult.filepickerClick == nsIFilePicker.returnCancel)
               return false;
 
           replacing = (dialogResult.filepickerClick == nsIFilePicker.returnReplace);
           urlstring = dialogResult.resultingURIString;
           tempLocalFile = dialogResult.resultingLocalFile;
  
       } catch (e) { alert(e + " " + e.lineNumber + " " + e.fileName); }
   }
   else {
       tempLocalFile = aFile;
   } // mustShowFileDialog
 
   var success = true;
   var ioService;
   try {
     // if somehow we didn't get a local file but we did get a uri, 
     // attempt to create the localfile if it's a "file" url
     var docURI;
     if (!tempLocalFile)
     {
       ioService = GetIOService();
       docURI = ioService.newURI(urlstring, editor.documentCharacterSet, null);
       
      if (docURI.schemeIs("file"))
      {
        var fileHandler = GetFileProtocolHandler();
        tempLocalFile = fileHandler.getFileFromURLSpec(urlstring).QueryInterface(Components.interfaces.nsILocalFile);
      }
    }

    success = OutputFile(theHtmlContent, tempLocalFile);
  }
  catch (e)
  {
     success = false;
     alert(e + " " + e.lineNumber + " " + e.fileName); 
  }
 
  if (success)
     return tempLocalFile;
  else
     return false;
}


