// vim: foldmethod=marker

//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2006
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//
//dojo.require("dojo.lang");

flock.dnd = {/*{{{*/
    // from the warmth of your bed
    // to the steps of your day
    // do not forget your life
}/*}}}*/

flock.dnd.Observer = function (aOpts) {/*{{{*/
    this._timers = {};
    flock.Base.call(this, aOpts);
}/*}}}*/
dojo.inherits(flock.dnd.Observer, flock.Base);
dojo.lang.extend(flock.dnd.Observer, {
    _timers: null,
    holdThreshold: 1250,

    // hold timer stuff
    // XXX andy: eventually I would like to get some of these timer
    //           things extracted into a more generic pattern/mixin
    startHoldTimer: function (aName, aEvent, aDragSession) {/*{{{*/
        aName = aName || "hold";
        if (this._timers[aName] != null) return;
        var inst=this;
        this._timers[aName] = setTimeout(
            function() { inst.onHoldOver( aEvent, aDragSession ); },
            this._holdThreshold );
    },/*}}}*/
    clearHoldTimer: function (aName) {/*{{{*/
        if (aName != null) {
            if (this._timers[aName] == null) return;
            clearTimeout(this._timers[aName]);
            this._timers[aName] = null;
        }
        else {
            for (t in this._timers) {
                this.clearHoldTimer(t);
            }
        }
    },/*}}}*/
    
    // DNDObserver interface
    /*{{{*/
    // Called if this item is dragged
	// ondraggesture = nsDragAndDrop.startDrag(event,DNDObserver)
	// onDragStart: function (aEvent, aXferData, aDragAction, aXferTarget) { },

	// Called when something is dragged into the item
	// Use this to set canDrop if the dragged item is useful    
	// ondragenter = nsDragAndDrop.dragEnter(event,DNDObserver)
	// onDragEnter: function (aEvent, aDragSession) { },

	// Called when something is dragged over the item
	// Use this to change the look of the element when an item
	// is being dragged over it
	// ondragover = nsDragAndDrop.dragOver(event,DNDObserver)
	// onDragOver: function (aEvent, aFlavour, aDragSession) { },
	
	// Called when an object has been dragged out from over the item
    // XXX: generally unreliable
	// Use this to return the look of the element to its original
	// ondragexit = nsDragAndDrop.dragExit((event,DNDObserver)
	// onDragExit: function (aEvent, aDragSession) { },

	// Called when an item is dropped on the item
	// Use this to handle things that are dropped on this
	// ondragdrop = nsDragAndDrop.drop(event, DNDObserver)
	// onDrop: function (aEvent, aXferData, aDragSession) { },
	
    // Called to get the flavours that this item accepts
    // should return a FlavourSet
	// getSupportedFlavours: function () { },
    /*}}}*/
});


/*
 * The drag and drop observer for the groups
 */
var DNDObserver = function (aOpts) {/*{{{*/
    XXX('DEPRECATED', 'this is the old DNDObserver, you should use flock.dnd.Observer instead');
    if (aOpts == null) { aOpts = {}; }
    dojo.lang.mixin(this, aOpts);
    if (aOpts['isDebug'] != null) {
        this.XXX("instantiating new DNDObserver in Debug mode");
    }
};/*}}}*/
DNDObserver.prototype = {
    // Set this to enable debug output
    isDebug: false,

    // Output debug info if isDebug is set
    XXX: function () {/*{{{*/
        if (this.isDebug) {
            XXX.apply(this, arguments);
        }
    },/*}}}*/
    
    // List of timers to hold on to
    _timers: {},

    // the number of milliseconds after which to trigger an 
    // onHoldOver event
    _holdThreshold: 1250,

    // shorcuts for starting and clearing hold timers
    startHoldTimer: function (aName, aEvent, aDragSession) {/*{{{*/
        aName = aName || "hold";
        this.XXX('startHoldTimer', aName);
        if (this._timers[aName] != null) return;
        var inst=this;
        this._timers[aName] = setTimeout(
            function() { inst.onHoldOver( aEvent, aDragSession ); },
            this._holdThreshold );
    },/*}}}*/
    clearHoldTimer: function (aName) {/*{{{*/
        if (aName != null) {
            this.XXX('clearHoldTimer', aName);
            if (this._timers[aName] == null) return;
            clearTimeout(this._timers[aName]);
            this._timers[aName] = null;
        }
        else {
            for (t in this._timers) {
                this.clearHoldTimer(t);
            }
        }
    },/*}}}*/
    
	// Called if this item is dragged
	// ondraggesture = nsDragAndDrop.startDrag(event,DNDObserver)
	onDragStart: function (aEvent, aXferData, aDragAction) {/*{{{*/
        this.XXX('onDragStart', aEvent.originalTarget.tagName, aDragAction);
        if (this.isDebug) {
            var data = new TransferData();
            data.addDataForFlavour("text/unicode", 
                                   aEvent.originalTarget.tagName);
            aXferData.data = data;
            return aXferData
        }
		// For elements we'll probably use this to reorder them
 
	},/*}}}*/

	// Called when something is dragged into the item
	// ondragenter = nsDragAndDrop.dragEnter(event,DNDObserver)
	onDragEnter: function (aEvent, aDragSession) {/*{{{*/
        this.XXX('onDragEnter', aEvent.originalTarget.tagName, aDragSession);
        if (this.isDebug) {
            aDragSession.canDrop = true;
        }
		// Use this to set canDrop if the dragged item is useful
		
	},/*}}}*/

	// Called when something is dragged over the item
	// ondragover = nsDragAndDrop.dragOver(event,DNDObserver)
	onDragOver: function (aEvent, aFlavour, aDragSession) {/*{{{*/
        this.XXX('onDragOver', aEvent.originalTarget.tagName, 
                 aFlavour, aDragSession);
        if (this.isDebug) {
            return true;
        }
		// Use this to change the look of the element when an item
		// is being dragged over it
	},/*}}}*/
	
	// Called when an object has been dragged out from over the item
	// ondragexit = nsDragAndDrop.dragExit((event,DNDObserver)
	onDragExit: function (aEvent, aDragSession) {/*{{{*/
        this.XXX('onDragExit', aEvent.originalTarget.tagName, aDragSession);
		// Use this to return the look of the element to its original
	},/*}}}*/

	// Called when an item is dropped on the item
	// ondragdrop = nsDragAndDrop.drop(event, DNDObserver)
	onDrop: function (aEvent, aXferData, aDragSession) {/*{{{*/
        this.XXX('onDrop', aEvent.originalTarget.tagName, aDragSession);
		// Use this to handle things that are dropped on this
	},/*}}}*/

	// Called to get the flavours that this item accepts
	getSupportedFlavours: function () {/*{{{*/
        this.XXX("getSupportedFlavours");
        if (this.isDebug) {
            var flavourSet = new FlavourSet();
            flavourSet.appendFlavour("text/unicode");
            this.XXX("flavourSet", flavourSet);
            return flavourSet;
        }
		// var flavourSet = new FlavourSet();
		// flavourSet.appendFlavour("text/x-moz-url");
		// return flavourSet;						   
	}/*}}}*/
}




function flock_getDragSession() {/*{{{*/
    return flock_getDragService().getCurrentSession();
}/*}}}*/


// Implements nsITransferable and then some.
var DNDTransfer = function (aOpts) {/*{{{*/
    if (!aOpts) aOpts = {};
    this.supportedFlavours = [];
    this.transferData = {};
    dojo.lang.mixin(this, aOpts);
    if (this.transferable) {
        this.fillFromTransferable(this.transferable);
    }
};/*}}}*/
dojo.lang.extend(DNDTransfer, {
    // nsITransferable
    kFlavorHasDataProvider: 0,
    converter: null,
    addDataFlavor: function (aFlavour) {/*{{{*/
        if (this.supportedFlavours.indexOf(aFlavour) != -1) return;
        this.supportedFlavours.push(aFlavour);
    },/*}}}*/
    flavorsTransferableCanExport: function () {/*{{{*/
        return this.supportedFlavours;
    },/*}}}*/
    flavorsTransferableCanImport: function () {/*{{{*/
        return this.supportedFlavours;
    },/*}}}*/
    getAnyTransferData: function (aFlavour, aData, aLen) {/*{{{*/
        var aInnerFlavour;
        for (var i=0; i<this.supportedFlavours.length; i++) {
            aInnerFlavour = this.supportedFlavours[i];
            if (this.transferData[aInnerFlavour] != null) {
                aFlavour.value = aInnerFlavour;
                aData.value = this.transferData[aInnerFlavour];
                aLen.value = aData.value.length;
                return;
            }
        }
    },/*}}}*/
    getTransferData: function (aFlavour, aData, aLen) {/*{{{*/
        if (aFlavour in this.transferData) {
            aData.value = this.transferData[aFlavour];
            aLen.value = aData.value.length;
        }
    },/*}}}*/
    isLargeDataSet: function () {/*{{{*/
        return false;
    },/*}}}*/
    removeDataFlavor: function (aFlavour) {/*{{{*/
        if (this.supportedFlavours.indexOf(aFlavour) != -1) {
            this.supportedFlavours.splice(this.supportedFlavours.indexOf(aFlavour));
            this.transferData[aFlavour] = null;
        }
    },/*}}}*/
    setTransferData: function (aFlavour, aData, aLen) {/*{{{*/
        if (this.supportedFlavours.indexOf(aFlavour) != -1) {
            this.transferData[aFlavour] = aData;
        }
    },/*}}}*/

    // the cool stuff
    isDebug: false,
    supportedFlavours: null,
    transferData: null,
    fillFromTransferable: function (aTransferable) {/*{{{*/
        this.supportedFlavours.forEach( function (aFlavour) {
            var dataObj = {};
            var len = {};
            try {
                aTransferable.getTransferData(aFlavour, dataObj, len);
                if (dataObj.value) {
                    this.setTransferData(aFlavour, dataObj.value, len.value);
                }
            } catch (e) {
                // did not manage to get anything for that flavour
                // pass
            }
        }, this);
    },/*}}}*/
    getBestFlavour: function (aData, aSupported) {/*{{{*/
        var aFlavour;
        for (var i=0; i<aSupported.length; i++) {
            aFlavour = aSupported[i];
            if (aFlavour in aData) {
                if (this.isDebug) XXX('getBestFlavour', aData, aSupported, aFlavour);
                return aFlavour;
            }
        }
    },/*}}}*/
    // parsing
    converters: {/*{{{*/
        "text/unicode": "string",
        "text/x-moz-url": "url"
    },/*}}}*/
    supportsTo: function (aType) {/*{{{*/
        if (this.getConverter(aType)) return true;
        return false;
    },/*}}}*/
    getConverter: function (aType) {/*{{{*/
        if (aType in this.converters) 
            aType = this.converters[aType];
        if (typeof aType == "string" && aType != "") 
            aType = this['to' + aType.substr(0, 1).toUpperCase() + aType.substr(1)];
        if (typeof aType != "function") 
            return;
        return aType;
    },/*}}}*/
    to: function (aType) {/*{{{*/
        var converter = this.getConverter(aType);
        if (!converter) dojo.raise("invalid conversion type");
        if (this.isDebug) XXX('to', aType, this.transferData);
        return converter.apply(this, [this.transferData]);
    },/*}}}*/
    // actual parsers
    toString: function (aData) {/*{{{*/
        if (typeof aData == "string") return aData;
        var flavour = this.getBestFlavour(
            aData,
            ["text/unicode", "text/x-moz-url", "application/x-moz-file"]
        );
        var data = aData[flavour];
        switch (flavour) {
            case "text/unicode":
            case "text/x-moz-url":
                return data.QueryInterface(Components.interfaces.nsISupportsString).toString();
            case "application/x-moz-file":
                return this.toUrl(aData).url;
        }
        return;
    },/*}}}*/
    toUrl: function (aData) {/*{{{*/
        var flavour = this.getBestFlavour(
            aData, 
            ["text/x-moz-url", "text/unicode", "application/x-moz-file"]
        );
        var data = aData[flavour];
        switch (flavour) {
            case "text/unicode":
                data = this.toString(aData);
                if (dojo.validate.isUrl(data)) return {url: data};
                break;
            case "text/x-moz-url":
                data = this.toString(aData);
                parts = data.split("\n");
                return {url: parts[0], name: parts[1]};
                break;
            case "application/x-moz-file":
                // thanks nsDragAndDrop.js
                var ioService = Components.classes["@mozilla.org/network/io-service;1"]
                    .getService(Components.interfaces.nsIIOService);
                var fileHandler = ioService.getProtocolHandler("file")
                    .QueryInterface(Components.interfaces.nsIFileProtocolHandler);
                data = fileHandler.getURLSpecFromFile(data);
                return {url: data};
        }
        return;
    }/*}}}*/
});



var DNDTransferSet = function (aOpts) {/*{{{*/
    if (!aOpts) aOpts = {};
    this.supportedFlavours = [];
    this.transferData = [];
    dojo.lang.mixin(this, aOpts);
    if (this.dragSession) {
        this.fillFromDragSession(this.dragSession);
    }
};/*}}}*/
dojo.lang.extend(DNDTransferSet, {
    supportedFlavours: null,
    transferData: null,
    fillFromDragSession: function (aDragSession) {/*{{{*/
        for (var i = 0; i < aDragSession.numDropItems; ++i) {
            var trans = Components.classes['@mozilla.org/widget/transferable;1']
                .createInstance(Components.interfaces.nsITransferable);
            this.supportedFlavours.forEach( function (aFlavour) { 
                trans.addDataFlavor(aFlavour); 
            });
            aDragSession.getData(trans, i);
            var transfer = new DNDTransfer({
                supportedFlavours: this.supportedFlavours,
                transferable: trans,
                isDebug: this.isDebug
            });
            this.transferData.push(transfer);       
        }
    },/*}}}*/
    getDataAs: function (aType, aIndex) {/*{{{*/
        if (aIndex) {
            return this.transferData[aIndex].to(aType);
        }
        var out = [];
        for (var i = 0; i < this.transferData.length; ++i) {
            var r = this.transferData[i].to(aType);
            if (this.isDebug) XXX('getDataAs', aType, r);
            if (r != null) out.push(r);
        }
        return out;
    }/*}}}*/
});
